# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UpdateAgentRequestConfigurationEndpoint(UniversalBaseModel):
    """
    When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.
    """

    url: str = pydantic.Field()
    """
    URL to call
    """

    headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Object of key-value pairs.
    """

    timeout_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    Timeout in milliseconds for the endpoint call.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .agent import Agent
from .agent_audio_format import AgentAudioFormat
from .agent_configuration_endpoint import AgentConfigurationEndpoint
from .agent_project import AgentProject
from .agent_template_variables_value import AgentTemplateVariablesValue
from .agent_tools_item import AgentToolsItem
from .assistant_chose_not_to_respond_payload import AssistantChoseNotToRespondPayload
from .assistant_ended_conversation_payload import AssistantEndedConversationPayload
from .audio_chunk_payload import AudioChunkPayload
from .audio_chunk_response_payload import AudioChunkResponsePayload
from .audio_finished_payload import AudioFinishedPayload
from .config_payload import ConfigPayload
from .config_payload_input_format import ConfigPayloadInputFormat
from .config_payload_output_format import ConfigPayloadOutputFormat
from .conversation import Conversation
from .conversation_analysis import ConversationAnalysis
from .conversation_created_payload import ConversationCreatedPayload
from .conversation_eval import ConversationEval
from .conversation_eval_prompt import ConversationEvalPrompt
from .conversation_evaluation import ConversationEvaluation
from .conversation_evaluation_result import ConversationEvaluationResult
from .conversation_evaluation_result_prompt import ConversationEvaluationResultPrompt
from .conversation_evaluation_result_result import ConversationEvaluationResultResult
from .conversation_extraction import ConversationExtraction
from .conversation_extraction_schema import ConversationExtractionSchema
from .conversation_item import ConversationItem
from .conversation_item_role import ConversationItemRole
from .create_agent_request import CreateAgentRequest
from .create_agent_request_audio_format import CreateAgentRequestAudioFormat
from .create_agent_request_configuration_endpoint import CreateAgentRequestConfigurationEndpoint
from .create_agent_request_template_variables_value import CreateAgentRequestTemplateVariablesValue
from .create_agent_request_tools_item import CreateAgentRequestToolsItem
from .dtmf_payload import DtmfPayload
from .error import Error
from .error_error import ErrorError
from .error_payload import ErrorPayload
from .error_payload_error import ErrorPayloadError
from .extraction_field import ExtractionField
from .extraction_field_type import ExtractionFieldType
from .extraction_schema import ExtractionSchema
from .input_cancelled_payload import InputCancelledPayload
from .input_text_payload import InputTextPayload
from .interrupted_response_payload import InterruptedResponsePayload
from .is_user_speaking_payload import IsUserSpeakingPayload
from .outbound_call_config import OutboundCallConfig
from .outbound_call_config_tools_item import OutboundCallConfigToolsItem
from .project import Project
from .project_default_agent import ProjectDefaultAgent
from .ready_to_start_conversation_payload import ReadyToStartConversationPayload
from .set_external_id_payload import SetExternalIdPayload
from .set_twilio_call_sid_payload import SetTwilioCallSidPayload
from .task import Task
from .tool import Tool
from .tool_call_interrupted_payload import ToolCallInterruptedPayload
from .tool_call_output_payload import ToolCallOutputPayload
from .tool_call_output_processed_payload import ToolCallOutputProcessedPayload
from .tool_call_output_processed_payload_tool import ToolCallOutputProcessedPayloadTool
from .tool_call_payload import ToolCallPayload
from .tool_execution_mode import ToolExecutionMode
from .tool_parameter import ToolParameter
from .tool_parameter_item_type import ToolParameterItemType
from .tool_parameter_type import ToolParameterType
from .tool_project import ToolProject
from .tool_type import ToolType
from .update_system_prompt_payload import UpdateSystemPromptPayload
from .user_finished_speaking_payload import UserFinishedSpeakingPayload
from .user_started_speaking_payload import UserStartedSpeakingPayload
from .voice import Voice

__all__ = [
    "Agent",
    "AgentAudioFormat",
    "AgentConfigurationEndpoint",
    "AgentProject",
    "AgentTemplateVariablesValue",
    "AgentToolsItem",
    "AssistantChoseNotToRespondPayload",
    "AssistantEndedConversationPayload",
    "AudioChunkPayload",
    "AudioChunkResponsePayload",
    "AudioFinishedPayload",
    "ConfigPayload",
    "ConfigPayloadInputFormat",
    "ConfigPayloadOutputFormat",
    "Conversation",
    "ConversationAnalysis",
    "ConversationCreatedPayload",
    "ConversationEval",
    "ConversationEvalPrompt",
    "ConversationEvaluation",
    "ConversationEvaluationResult",
    "ConversationEvaluationResultPrompt",
    "ConversationEvaluationResultResult",
    "ConversationExtraction",
    "ConversationExtractionSchema",
    "ConversationItem",
    "ConversationItemRole",
    "CreateAgentRequest",
    "CreateAgentRequestAudioFormat",
    "CreateAgentRequestConfigurationEndpoint",
    "CreateAgentRequestTemplateVariablesValue",
    "CreateAgentRequestToolsItem",
    "DtmfPayload",
    "Error",
    "ErrorError",
    "ErrorPayload",
    "ErrorPayloadError",
    "ExtractionField",
    "ExtractionFieldType",
    "ExtractionSchema",
    "InputCancelledPayload",
    "InputTextPayload",
    "InterruptedResponsePayload",
    "IsUserSpeakingPayload",
    "OutboundCallConfig",
    "OutboundCallConfigToolsItem",
    "Project",
    "ProjectDefaultAgent",
    "ReadyToStartConversationPayload",
    "SetExternalIdPayload",
    "SetTwilioCallSidPayload",
    "Task",
    "Tool",
    "ToolCallInterruptedPayload",
    "ToolCallOutputPayload",
    "ToolCallOutputProcessedPayload",
    "ToolCallOutputProcessedPayloadTool",
    "ToolCallPayload",
    "ToolExecutionMode",
    "ToolParameter",
    "ToolParameterItemType",
    "ToolParameterType",
    "ToolProject",
    "ToolType",
    "UpdateSystemPromptPayload",
    "UserFinishedSpeakingPayload",
    "UserStartedSpeakingPayload",
    "Voice",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .conversations_list_response_conversation import ConversationsListResponseConversation
from .conversations_list_response_conversations import ConversationsListResponseConversations

ConversationsListResponse = typing.Union[ConversationsListResponseConversations, ConversationsListResponseConversation]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawProjectsClient, RawProjectsClient
from .types.projects_create_eval_prompt_response import ProjectsCreateEvalPromptResponse
from .types.projects_create_response import ProjectsCreateResponse
from .types.projects_delete_response import ProjectsDeleteResponse
from .types.projects_get_response import ProjectsGetResponse
from .types.projects_list_eval_prompts_response import ProjectsListEvalPromptsResponse
from .types.projects_list_response import ProjectsListResponse
from .types.projects_update_response import ProjectsUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProjectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProjectsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawProjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProjectsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ProjectsListResponse:
        """
        Returns all projects in a workspace.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsListResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.projects.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def create(self, *, name: str, request_options: typing.Optional[RequestOptions] = None) -> ProjectsCreateResponse:
        """
        Creates a new project in a workspace.

        Parameters
        ----------
        name : str
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsCreateResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.projects.create(
            name="customer-support",
        )
        """
        _response = self._raw_client.create(name=name, request_options=request_options)
        return _response.data

    def get(self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ProjectsGetResponse:
        """
        Returns a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsGetResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.projects.get(
            name_or_id="nameOrId",
        )
        """
        _response = self._raw_client.get(name_or_id, request_options=request_options)
        return _response.data

    def delete(
        self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsDeleteResponse:
        """
        Deletes a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsDeleteResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.projects.delete(
            name_or_id="nameOrId",
        )
        """
        _response = self._raw_client.delete(name_or_id, request_options=request_options)
        return _response.data

    def update(
        self,
        name_or_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        default_agent: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectsUpdateResponse:
        """
        Updates a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to update.

        name : typing.Optional[str]
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        default_agent : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsUpdateResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.projects.update(
            name_or_id="nameOrId",
            name="updated-customer-support",
            default_agent="another-agent",
        )
        """
        _response = self._raw_client.update(
            name_or_id, name=name, default_agent=default_agent, request_options=request_options
        )
        return _response.data

    def list_eval_prompts(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsListEvalPromptsResponse:
        """
        Returns all conversation evaluation prompts for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsListEvalPromptsResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.projects.list_eval_prompts(
            id="id",
        )
        """
        _response = self._raw_client.list_eval_prompts(id, request_options=request_options)
        return _response.data

    def create_eval_prompt(
        self, id: str, *, name: str, prompt: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsCreateEvalPromptResponse:
        """
        Creates a new conversation evaluation prompt for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        name : str
            A useful name for referring to this prompt.

        prompt : str
            Actual evaluation prompt text to evaluate conversations with.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsCreateEvalPromptResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.projects.create_eval_prompt(
            id="id",
            name="test_prompt",
            prompt="The assistant used the word chocolate in the conversation",
        )
        """
        _response = self._raw_client.create_eval_prompt(id, name=name, prompt=prompt, request_options=request_options)
        return _response.data


class AsyncProjectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProjectsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawProjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProjectsClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ProjectsListResponse:
        """
        Returns all projects in a workspace.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsListResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.projects.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def create(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsCreateResponse:
        """
        Creates a new project in a workspace.

        Parameters
        ----------
        name : str
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsCreateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.projects.create(
                name="customer-support",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(name=name, request_options=request_options)
        return _response.data

    async def get(
        self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsGetResponse:
        """
        Returns a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsGetResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.projects.get(
                name_or_id="nameOrId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(name_or_id, request_options=request_options)
        return _response.data

    async def delete(
        self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsDeleteResponse:
        """
        Deletes a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsDeleteResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.projects.delete(
                name_or_id="nameOrId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(name_or_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        name_or_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        default_agent: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectsUpdateResponse:
        """
        Updates a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to update.

        name : typing.Optional[str]
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        default_agent : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsUpdateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.projects.update(
                name_or_id="nameOrId",
                name="updated-customer-support",
                default_agent="another-agent",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            name_or_id, name=name, default_agent=default_agent, request_options=request_options
        )
        return _response.data

    async def list_eval_prompts(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsListEvalPromptsResponse:
        """
        Returns all conversation evaluation prompts for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsListEvalPromptsResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.projects.list_eval_prompts(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_eval_prompts(id, request_options=request_options)
        return _response.data

    async def create_eval_prompt(
        self, id: str, *, name: str, prompt: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectsCreateEvalPromptResponse:
        """
        Creates a new conversation evaluation prompt for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        name : str
            A useful name for referring to this prompt.

        prompt : str
            Actual evaluation prompt text to evaluate conversations with.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectsCreateEvalPromptResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.projects.create_eval_prompt(
                id="id",
                name="test_prompt",
                prompt="The assistant used the word chocolate in the conversation",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_eval_prompt(
            id, name=name, prompt=prompt, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.conversation_evaluation_result import ConversationEvaluationResult
from ..types.outbound_call_config import OutboundCallConfig
from .raw_client import AsyncRawConversationsClient, RawConversationsClient
from .types.conversations_cancel_response import ConversationsCancelResponse
from .types.conversations_extract_data_response import ConversationsExtractDataResponse
from .types.conversations_get_analysis_response import ConversationsGetAnalysisResponse
from .types.conversations_get_response import ConversationsGetResponse
from .types.conversations_list_evaluations_response import ConversationsListEvaluationsResponse
from .types.conversations_list_extractions_response import ConversationsListExtractionsResponse
from .types.conversations_list_response import ConversationsListResponse
from .types.conversations_outbound_call_response import ConversationsOutboundCallResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConversationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConversationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConversationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConversationsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        project: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        duration_min: typing.Optional[int] = None,
        duration_max: typing.Optional[int] = None,
        started_at_min: typing.Optional[str] = None,
        started_at_max: typing.Optional[str] = None,
        before: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsListResponse:
        """
        Returns conversations with optional filtering.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list conversations for.

        external_id : typing.Optional[str]
            Filter by external ID to get a specific conversation.

        duration_min : typing.Optional[int]
            Minimum duration in seconds.

        duration_max : typing.Optional[int]
            Maximum duration in seconds.

        started_at_min : typing.Optional[str]
            Minimum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        started_at_max : typing.Optional[str]
            Maximum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        before : typing.Optional[str]
            Cursor for pagination (before).

        after : typing.Optional[str]
            Cursor for pagination (after).

        limit : typing.Optional[int]
            Maximum number of conversations to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.list()
        """
        _response = self._raw_client.list(
            project=project,
            external_id=external_id,
            duration_min=duration_min,
            duration_max=duration_max,
            started_at_min=started_at_min,
            started_at_max=started_at_max,
            before=before,
            after=after,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ConversationsGetResponse:
        """
        Returns a conversation by ID.

        Parameters
        ----------
        id : str
            The ID of the conversation to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def cancel(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsCancelResponse:
        """
        Cancels an active conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsCancelResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.cancel(
            id="id",
        )
        """
        _response = self._raw_client.cancel(id, request_options=request_options)
        return _response.data

    def get_analysis(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsGetAnalysisResponse:
        """
        Returns an analysis of the specified conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to analyze.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetAnalysisResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.get_analysis(
            id="id",
        )
        """
        _response = self._raw_client.get_analysis(id, request_options=request_options)
        return _response.data

    def list_extractions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListExtractionsResponse:
        """
        Returns all extractions for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get extractions for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListExtractionsResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.list_extractions(
            id="id",
        )
        """
        _response = self._raw_client.list_extractions(id, request_options=request_options)
        return _response.data

    def extract_data(
        self, id: str, *, schema_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsExtractDataResponse:
        """
        Extracts data from a conversation using a schema.

        Parameters
        ----------
        id : str
            The ID of the conversation to extract data from.

        schema_id : str
            ID of the extraction schema to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsExtractDataResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.extract_data(
            id="id",
            schema_id="conv_extract_schema_6458e4ac-533c-4bdf-8e6d-c2f06f87fd5c",
        )
        """
        _response = self._raw_client.extract_data(id, schema_id=schema_id, request_options=request_options)
        return _response.data

    def list_evaluations(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListEvaluationsResponse:
        """
        Returns all evaluations for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get evaluations for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListEvaluationsResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.list_evaluations(
            id="id",
        )
        """
        _response = self._raw_client.list_evaluations(id, request_options=request_options)
        return _response.data

    def evaluate(
        self, id: str, *, prompt_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationEvaluationResult:
        """
        Evaluates a conversation using an evaluation prompt.

        Parameters
        ----------
        id : str
            The ID of the conversation to evaluate.

        prompt_id : str
            ID of the evaluation prompt to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationEvaluationResult
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.evaluate(
            id="id",
            prompt_id="conv_eval_prompt_d7cfe45d-35db-4ef6-a254-81ab1da76ce0",
        )
        """
        _response = self._raw_client.evaluate(id, prompt_id=prompt_id, request_options=request_options)
        return _response.data

    def outbound_call(
        self,
        *,
        to_phone_number: str,
        config: typing.Optional[OutboundCallConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsOutboundCallResponse:
        """
        Initiates a call to a given phone number using Phonic's Twilio account.

        Parameters
        ----------
        to_phone_number : str
            The phone number to call in E.164 format.

        config : typing.Optional[OutboundCallConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsOutboundCallResponse
            Success response

        Examples
        --------
        from phonic import OutboundCallConfig, Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.outbound_call(
            to_phone_number="+19189397081",
            config=OutboundCallConfig(
                agent="support-agent",
                welcome_message="Hi {{customer_name}}. How can I help you today?",
                system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
                template_variables={"customer_name": "David", "subject": "Chess"},
                voice_id="sarah",
                no_input_poke_sec=30,
                no_input_poke_text="Are you still there?",
                no_input_end_conversation_sec=180,
                boosted_keywords=["Load ID", "dispatch"],
                tools=[],
            ),
        )
        """
        _response = self._raw_client.outbound_call(
            to_phone_number=to_phone_number, config=config, request_options=request_options
        )
        return _response.data

    def summarize(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.conversations.summarize(
            id="id",
        )
        """
        _response = self._raw_client.summarize(id, request_options=request_options)
        return _response.data


class AsyncConversationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConversationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConversationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConversationsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        project: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        duration_min: typing.Optional[int] = None,
        duration_max: typing.Optional[int] = None,
        started_at_min: typing.Optional[str] = None,
        started_at_max: typing.Optional[str] = None,
        before: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsListResponse:
        """
        Returns conversations with optional filtering.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list conversations for.

        external_id : typing.Optional[str]
            Filter by external ID to get a specific conversation.

        duration_min : typing.Optional[int]
            Minimum duration in seconds.

        duration_max : typing.Optional[int]
            Maximum duration in seconds.

        started_at_min : typing.Optional[str]
            Minimum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        started_at_max : typing.Optional[str]
            Maximum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        before : typing.Optional[str]
            Cursor for pagination (before).

        after : typing.Optional[str]
            Cursor for pagination (after).

        limit : typing.Optional[int]
            Maximum number of conversations to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            project=project,
            external_id=external_id,
            duration_min=duration_min,
            duration_max=duration_max,
            started_at_min=started_at_min,
            started_at_max=started_at_max,
            before=before,
            after=after,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsGetResponse:
        """
        Returns a conversation by ID.

        Parameters
        ----------
        id : str
            The ID of the conversation to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def cancel(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsCancelResponse:
        """
        Cancels an active conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsCancelResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.cancel(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel(id, request_options=request_options)
        return _response.data

    async def get_analysis(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsGetAnalysisResponse:
        """
        Returns an analysis of the specified conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to analyze.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetAnalysisResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.get_analysis(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_analysis(id, request_options=request_options)
        return _response.data

    async def list_extractions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListExtractionsResponse:
        """
        Returns all extractions for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get extractions for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListExtractionsResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.list_extractions(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_extractions(id, request_options=request_options)
        return _response.data

    async def extract_data(
        self, id: str, *, schema_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsExtractDataResponse:
        """
        Extracts data from a conversation using a schema.

        Parameters
        ----------
        id : str
            The ID of the conversation to extract data from.

        schema_id : str
            ID of the extraction schema to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsExtractDataResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.extract_data(
                id="id",
                schema_id="conv_extract_schema_6458e4ac-533c-4bdf-8e6d-c2f06f87fd5c",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.extract_data(id, schema_id=schema_id, request_options=request_options)
        return _response.data

    async def list_evaluations(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListEvaluationsResponse:
        """
        Returns all evaluations for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get evaluations for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListEvaluationsResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.list_evaluations(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_evaluations(id, request_options=request_options)
        return _response.data

    async def evaluate(
        self, id: str, *, prompt_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationEvaluationResult:
        """
        Evaluates a conversation using an evaluation prompt.

        Parameters
        ----------
        id : str
            The ID of the conversation to evaluate.

        prompt_id : str
            ID of the evaluation prompt to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationEvaluationResult
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.evaluate(
                id="id",
                prompt_id="conv_eval_prompt_d7cfe45d-35db-4ef6-a254-81ab1da76ce0",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.evaluate(id, prompt_id=prompt_id, request_options=request_options)
        return _response.data

    async def outbound_call(
        self,
        *,
        to_phone_number: str,
        config: typing.Optional[OutboundCallConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsOutboundCallResponse:
        """
        Initiates a call to a given phone number using Phonic's Twilio account.

        Parameters
        ----------
        to_phone_number : str
            The phone number to call in E.164 format.

        config : typing.Optional[OutboundCallConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsOutboundCallResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic, OutboundCallConfig

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.outbound_call(
                to_phone_number="+19189397081",
                config=OutboundCallConfig(
                    agent="support-agent",
                    welcome_message="Hi {{customer_name}}. How can I help you today?",
                    system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
                    template_variables={"customer_name": "David", "subject": "Chess"},
                    voice_id="sarah",
                    no_input_poke_sec=30,
                    no_input_poke_text="Are you still there?",
                    no_input_end_conversation_sec=180,
                    boosted_keywords=["Load ID", "dispatch"],
                    tools=[],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.outbound_call(
            to_phone_number=to_phone_number, config=config, request_options=request_options
        )
        return _response.data

    async def summarize(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.conversations.summarize(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.summarize(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .create_tool_request_execution_mode import CreateToolRequestExecutionMode
from .create_tool_request_type import CreateToolRequestType
from .tools_create_response import ToolsCreateResponse
from .tools_delete_response import ToolsDeleteResponse
from .tools_get_response import ToolsGetResponse
from .tools_list_response import ToolsListResponse
from .tools_update_response import ToolsUpdateResponse
from .update_tool_request_execution_mode import UpdateToolRequestExecutionMode
from .update_tool_request_type import UpdateToolRequestType

__all__ = [
    "CreateToolRequestExecutionMode",
    "CreateToolRequestType",
    "ToolsCreateResponse",
    "ToolsDeleteResponse",
    "ToolsGetResponse",
    "ToolsListResponse",
    "ToolsUpdateResponse",
    "UpdateToolRequestExecutionMode",
    "UpdateToolRequestType",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .projects_create_eval_prompt_response import ProjectsCreateEvalPromptResponse
from .projects_create_response import ProjectsCreateResponse
from .projects_delete_response import ProjectsDeleteResponse
from .projects_get_response import ProjectsGetResponse
from .projects_list_eval_prompts_response import ProjectsListEvalPromptsResponse
from .projects_list_response import ProjectsListResponse
from .projects_update_response import ProjectsUpdateResponse

__all__ = [
    "ProjectsCreateEvalPromptResponse",
    "ProjectsCreateResponse",
    "ProjectsDeleteResponse",
    "ProjectsGetResponse",
    "ProjectsListEvalPromptsResponse",
    "ProjectsListResponse",
    "ProjectsUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .conversation_item_role import ConversationItemRole


class ConversationItem(UniversalBaseModel):
    item_idx: int = pydantic.Field()
    """
    Index of the item in the conversation.
    """

    role: ConversationItemRole = pydantic.Field()
    """
    Who spoke in this turn.
    """

    live_transcript: str = pydantic.Field()
    """
    Live transcript of this turn.
    """

    post_call_transcript: typing.Optional[str] = pydantic.Field(default=None)
    """
    Post-call processed transcript.
    """

    duration_ms: float = pydantic.Field()
    """
    Duration of this turn in milliseconds.
    """

    started_at: dt.datetime = pydantic.Field()
    """
    When this turn started.
    """

    system_prompt: typing.Optional[str] = pydantic.Field(default=None)
    """
    System prompt used (assistant only).
    """

    voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Voice ID used (assistant only).
    """

    audio_speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Audio speed used (assistant only).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawVoicesClient, RawVoicesClient
from .types.voices_get_response import VoicesGetResponse
from .types.voices_list_response import VoicesListResponse


class VoicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVoicesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVoicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVoicesClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> VoicesListResponse:
        """
        Returns all available voices for a model.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoicesListResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.voices.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> VoicesGetResponse:
        """
        Returns a voice by ID.

        Parameters
        ----------
        id : str
            The ID of the voice to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoicesGetResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.voices.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data


class AsyncVoicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVoicesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVoicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVoicesClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> VoicesListResponse:
        """
        Returns all available voices for a model.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoicesListResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.voices.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> VoicesGetResponse:
        """
        Returns a voice by ID.

        Parameters
        ----------
        id : str
            The ID of the voice to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoicesGetResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.voices.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from contextlib import asynccontextmanager, contextmanager

import httpx
import websockets.exceptions
import websockets.sync.client as websockets_sync_client
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .socket_client import AsyncStsSocketClient, StsSocketClient

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore


class RawStsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextmanager
    def connect(
        self,
        *,
        downstream_websocket_url: typing.Optional[str] = None,
        authorization: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[StsSocketClient]:
        """
        Main STS WebSocket channel for real-time voice conversations

        Parameters
        ----------
        downstream_websocket_url : typing.Optional[str]
            Custom downstream WebSocket URL

        authorization : typing.Optional[str]
            API key for authentication. Format: 'Bearer PHONIC_API_KEY'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StsSocketClient
        """
        ws_url = self._client_wrapper.get_environment().production + "/v1/sts/ws"
        query_params = httpx.QueryParams()
        if downstream_websocket_url is not None:
            query_params = query_params.add("downstream_websocket_url", downstream_websocket_url)
        ws_url = ws_url + f"?{query_params}"
        headers = self._client_wrapper.get_headers()
        if authorization is not None:
            headers["Authorization"] = str(authorization)
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield StsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncRawStsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @asynccontextmanager
    async def connect(
        self,
        *,
        downstream_websocket_url: typing.Optional[str] = None,
        authorization: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncStsSocketClient]:
        """
        Main STS WebSocket channel for real-time voice conversations

        Parameters
        ----------
        downstream_websocket_url : typing.Optional[str]
            Custom downstream WebSocket URL

        authorization : typing.Optional[str]
            API key for authentication. Format: 'Bearer PHONIC_API_KEY'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncStsSocketClient
        """
        ws_url = self._client_wrapper.get_environment().production + "/v1/sts/ws"
        query_params = httpx.QueryParams()
        if downstream_websocket_url is not None:
            query_params = query_params.add("downstream_websocket_url", downstream_websocket_url)
        ws_url = ws_url + f"?{query_params}"
        headers = self._client_wrapper.get_headers()
        if authorization is not None:
            headers["Authorization"] = str(authorization)
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncStsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ToolCallPayload(UniversalBaseModel):
    type: typing.Literal["tool_call"] = "tool_call"
    tool_call_id: str = pydantic.Field()
    """
    Unique ID for this tool call
    """

    tool_name: str = pydantic.Field()
    """
    Name of the tool to execute
    """

    parameters: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Parameters for tool execution
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .voices_get_response import VoicesGetResponse
from .voices_list_response import VoicesListResponse

__all__ = ["VoicesGetResponse", "VoicesListResponse"]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .agents_create_response import AgentsCreateResponse
from .agents_delete_response import AgentsDeleteResponse
from .agents_get_response import AgentsGetResponse
from .agents_list_response import AgentsListResponse
from .agents_update_response import AgentsUpdateResponse
from .agents_upsert_response import AgentsUpsertResponse
from .update_agent_request_audio_format import UpdateAgentRequestAudioFormat
from .update_agent_request_configuration_endpoint import UpdateAgentRequestConfigurationEndpoint
from .update_agent_request_template_variables_value import UpdateAgentRequestTemplateVariablesValue
from .update_agent_request_tools_item import UpdateAgentRequestToolsItem

__all__ = [
    "AgentsCreateResponse",
    "AgentsDeleteResponse",
    "AgentsGetResponse",
    "AgentsListResponse",
    "AgentsUpdateResponse",
    "AgentsUpsertResponse",
    "UpdateAgentRequestAudioFormat",
    "UpdateAgentRequestConfigurationEndpoint",
    "UpdateAgentRequestTemplateVariablesValue",
    "UpdateAgentRequestToolsItem",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ConversationsCancelResponse,
    ConversationsExtractDataResponse,
    ConversationsGetAnalysisResponse,
    ConversationsGetResponse,
    ConversationsListEvaluationsResponse,
    ConversationsListExtractionsResponse,
    ConversationsListResponse,
    ConversationsListResponseConversation,
    ConversationsListResponseConversations,
    ConversationsOutboundCallResponse,
)

__all__ = [
    "ConversationsCancelResponse",
    "ConversationsExtractDataResponse",
    "ConversationsGetAnalysisResponse",
    "ConversationsGetResponse",
    "ConversationsListEvaluationsResponse",
    "ConversationsListExtractionsResponse",
    "ConversationsListResponse",
    "ConversationsListResponseConversation",
    "ConversationsListResponseConversations",
    "ConversationsOutboundCallResponse",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ExtractionSchemasCreateResponse,
    ExtractionSchemasDeleteResponse,
    ExtractionSchemasGetResponse,
    ExtractionSchemasListResponse,
    ExtractionSchemasUpdateResponse,
)

__all__ = [
    "ExtractionSchemasCreateResponse",
    "ExtractionSchemasDeleteResponse",
    "ExtractionSchemasGetResponse",
    "ExtractionSchemasListResponse",
    "ExtractionSchemasUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ProjectsCreateEvalPromptResponse,
    ProjectsCreateResponse,
    ProjectsDeleteResponse,
    ProjectsGetResponse,
    ProjectsListEvalPromptsResponse,
    ProjectsListResponse,
    ProjectsUpdateResponse,
)

__all__ = [
    "ProjectsCreateEvalPromptResponse",
    "ProjectsCreateResponse",
    "ProjectsDeleteResponse",
    "ProjectsGetResponse",
    "ProjectsListEvalPromptsResponse",
    "ProjectsListResponse",
    "ProjectsUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .conversations_cancel_response import ConversationsCancelResponse
from .conversations_extract_data_response import ConversationsExtractDataResponse
from .conversations_get_analysis_response import ConversationsGetAnalysisResponse
from .conversations_get_response import ConversationsGetResponse
from .conversations_list_evaluations_response import ConversationsListEvaluationsResponse
from .conversations_list_extractions_response import ConversationsListExtractionsResponse
from .conversations_list_response import ConversationsListResponse
from .conversations_list_response_conversation import ConversationsListResponseConversation
from .conversations_list_response_conversations import ConversationsListResponseConversations
from .conversations_outbound_call_response import ConversationsOutboundCallResponse

__all__ = [
    "ConversationsCancelResponse",
    "ConversationsExtractDataResponse",
    "ConversationsGetAnalysisResponse",
    "ConversationsGetResponse",
    "ConversationsListEvaluationsResponse",
    "ConversationsListExtractionsResponse",
    "ConversationsListResponse",
    "ConversationsListResponseConversation",
    "ConversationsListResponseConversations",
    "ConversationsOutboundCallResponse",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CreateToolRequestExecutionMode,
    CreateToolRequestType,
    ToolsCreateResponse,
    ToolsDeleteResponse,
    ToolsGetResponse,
    ToolsListResponse,
    ToolsUpdateResponse,
    UpdateToolRequestExecutionMode,
    UpdateToolRequestType,
)

__all__ = [
    "CreateToolRequestExecutionMode",
    "CreateToolRequestType",
    "ToolsCreateResponse",
    "ToolsDeleteResponse",
    "ToolsGetResponse",
    "ToolsListResponse",
    "ToolsUpdateResponse",
    "UpdateToolRequestExecutionMode",
    "UpdateToolRequestType",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_agent_request_audio_format import CreateAgentRequestAudioFormat
from ..types.create_agent_request_configuration_endpoint import CreateAgentRequestConfigurationEndpoint
from ..types.create_agent_request_template_variables_value import CreateAgentRequestTemplateVariablesValue
from ..types.create_agent_request_tools_item import CreateAgentRequestToolsItem
from ..types.task import Task
from .raw_client import AsyncRawAgentsClient, RawAgentsClient
from .types.agents_create_response import AgentsCreateResponse
from .types.agents_delete_response import AgentsDeleteResponse
from .types.agents_get_response import AgentsGetResponse
from .types.agents_list_response import AgentsListResponse
from .types.agents_update_response import AgentsUpdateResponse
from .types.agents_upsert_response import AgentsUpsertResponse
from .types.update_agent_request_audio_format import UpdateAgentRequestAudioFormat
from .types.update_agent_request_configuration_endpoint import UpdateAgentRequestConfigurationEndpoint
from .types.update_agent_request_template_variables_value import UpdateAgentRequestTemplateVariablesValue
from .types.update_agent_request_tools_item import UpdateAgentRequestToolsItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AgentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAgentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAgentsClient
        """
        return self._raw_client

    def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AgentsListResponse:
        """
        Returns all agents in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list agents for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsListResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.agents.list()
        """
        _response = self._raw_client.list(project=project, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]] = OMIT,
        tasks: typing.Optional[typing.Sequence[Task]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpoint] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsCreateResponse:
        """
        Creates a new agent in a project.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project to create the agent in.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[Task]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpoint]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsCreateResponse
            Success response

        Examples
        --------
        from phonic import (
            CreateAgentRequestConfigurationEndpoint,
            CreateAgentRequestTemplateVariablesValue,
            Phonic,
        )

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.agents.create(
            name="support-agent",
            timezone="America/Los_Angeles",
            voice_id="sarah",
            audio_speed=1.0,
            welcome_message="Hi {{customer_name}}. How can I help you today?",
            system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
            template_variables={
                "customer_name": CreateAgentRequestTemplateVariablesValue(),
                "subject": CreateAgentRequestTemplateVariablesValue(
                    default_value="Chess",
                ),
            },
            tools=[],
            no_input_poke_sec=30,
            no_input_poke_text="Are you still there?",
            boosted_keywords=["Load ID", "dispatch"],
            configuration_endpoint=CreateAgentRequestConfigurationEndpoint(
                url="https://api.example.com/config",
                headers={"Authorization": "Bearer token123"},
                timeout_ms=7000,
            ),
        )
        """
        _response = self._raw_client.create(
            name=name,
            project=project,
            phone_number=phone_number,
            timezone=timezone,
            voice_id=voice_id,
            audio_format=audio_format,
            audio_speed=audio_speed,
            welcome_message=welcome_message,
            system_prompt=system_prompt,
            template_variables=template_variables,
            tools=tools,
            tasks=tasks,
            no_input_poke_sec=no_input_poke_sec,
            no_input_poke_text=no_input_poke_text,
            no_input_end_conversation_sec=no_input_end_conversation_sec,
            boosted_keywords=boosted_keywords,
            configuration_endpoint=configuration_endpoint,
            request_options=request_options,
        )
        return _response.data

    def upsert(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]] = OMIT,
        tasks: typing.Optional[typing.Sequence[Task]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpoint] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsUpsertResponse:
        """
        Upserts an agent by name. If an agent with the same name already exists, it will be updated. Otherwise, it will be created.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project containing the agent.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[Task]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpoint]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsUpsertResponse
            Success response

        Examples
        --------
        from phonic import (
            CreateAgentRequestConfigurationEndpoint,
            CreateAgentRequestTemplateVariablesValue,
            Phonic,
        )

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.agents.upsert(
            name="support-agent",
            phone_number="assign-automatically",
            timezone="America/Los_Angeles",
            voice_id="sarah",
            audio_speed=1.0,
            welcome_message="Hi {{customer_name}}. How can I help you today?",
            system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
            template_variables={
                "customer_name": CreateAgentRequestTemplateVariablesValue(),
                "subject": CreateAgentRequestTemplateVariablesValue(
                    default_value="Chess",
                ),
            },
            tools=[],
            no_input_poke_sec=30,
            no_input_poke_text="Are you still there?",
            boosted_keywords=["Load ID", "dispatch"],
            configuration_endpoint=CreateAgentRequestConfigurationEndpoint(
                url="https://api.example.com/config",
                headers={"Authorization": "Bearer token123"},
                timeout_ms=7000,
            ),
        )
        """
        _response = self._raw_client.upsert(
            name=name,
            project=project,
            phone_number=phone_number,
            timezone=timezone,
            voice_id=voice_id,
            audio_format=audio_format,
            audio_speed=audio_speed,
            welcome_message=welcome_message,
            system_prompt=system_prompt,
            template_variables=template_variables,
            tools=tools,
            tasks=tasks,
            no_input_poke_sec=no_input_poke_sec,
            no_input_poke_text=no_input_poke_text,
            no_input_end_conversation_sec=no_input_end_conversation_sec,
            boosted_keywords=boosted_keywords,
            configuration_endpoint=configuration_endpoint,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsGetResponse:
        """
        Returns an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to get.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsGetResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.agents.get(
            name_or_id="nameOrId",
        )
        """
        _response = self._raw_client.get(name_or_id, project=project, request_options=request_options)
        return _response.data

    def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsDeleteResponse:
        """
        Deletes an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to delete.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsDeleteResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.agents.delete(
            name_or_id="nameOrId",
        )
        """
        _response = self._raw_client.delete(name_or_id, project=project, request_options=request_options)
        return _response.data

    def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[UpdateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValue]] = OMIT,
        tools: typing.Optional[typing.Sequence[UpdateAgentRequestToolsItem]] = OMIT,
        tasks: typing.Optional[typing.Sequence[Task]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[UpdateAgentRequestConfigurationEndpoint] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsUpdateResponse:
        """
        Updates an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to update.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[UpdateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValue]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[UpdateAgentRequestToolsItem]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[Task]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[UpdateAgentRequestConfigurationEndpoint]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsUpdateResponse
            Success response

        Examples
        --------
        from phonic import Phonic
        from phonic.agents import (
            UpdateAgentRequestConfigurationEndpoint,
            UpdateAgentRequestTemplateVariablesValue,
        )

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.agents.update(
            name_or_id="nameOrId",
            name="updated-support-agent",
            phone_number="assign-automatically",
            timezone="America/Los_Angeles",
            voice_id="sarah",
            audio_speed=1.0,
            welcome_message="Hi {{customer_name}}. How can I help you today?",
            system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
            template_variables={
                "customer_name": UpdateAgentRequestTemplateVariablesValue(),
                "subject": UpdateAgentRequestTemplateVariablesValue(
                    default_value="Chess",
                ),
            },
            tools=[],
            no_input_poke_sec=30,
            no_input_poke_text="Are you still there?",
            boosted_keywords=["Load ID", "dispatch"],
            configuration_endpoint=UpdateAgentRequestConfigurationEndpoint(
                url="https://api.example.com/config",
                headers={"Authorization": "Bearer token123"},
                timeout_ms=7000,
            ),
        )
        """
        _response = self._raw_client.update(
            name_or_id,
            project=project,
            name=name,
            phone_number=phone_number,
            timezone=timezone,
            voice_id=voice_id,
            audio_format=audio_format,
            audio_speed=audio_speed,
            welcome_message=welcome_message,
            system_prompt=system_prompt,
            template_variables=template_variables,
            tools=tools,
            tasks=tasks,
            no_input_poke_sec=no_input_poke_sec,
            no_input_poke_text=no_input_poke_text,
            no_input_end_conversation_sec=no_input_end_conversation_sec,
            boosted_keywords=boosted_keywords,
            configuration_endpoint=configuration_endpoint,
            request_options=request_options,
        )
        return _response.data


class AsyncAgentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAgentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAgentsClient
        """
        return self._raw_client

    async def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AgentsListResponse:
        """
        Returns all agents in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list agents for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsListResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project=project, request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]] = OMIT,
        tasks: typing.Optional[typing.Sequence[Task]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpoint] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsCreateResponse:
        """
        Creates a new agent in a project.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project to create the agent in.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[Task]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpoint]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsCreateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import (
            AsyncPhonic,
            CreateAgentRequestConfigurationEndpoint,
            CreateAgentRequestTemplateVariablesValue,
        )

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.create(
                name="support-agent",
                timezone="America/Los_Angeles",
                voice_id="sarah",
                audio_speed=1.0,
                welcome_message="Hi {{customer_name}}. How can I help you today?",
                system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
                template_variables={
                    "customer_name": CreateAgentRequestTemplateVariablesValue(),
                    "subject": CreateAgentRequestTemplateVariablesValue(
                        default_value="Chess",
                    ),
                },
                tools=[],
                no_input_poke_sec=30,
                no_input_poke_text="Are you still there?",
                boosted_keywords=["Load ID", "dispatch"],
                configuration_endpoint=CreateAgentRequestConfigurationEndpoint(
                    url="https://api.example.com/config",
                    headers={"Authorization": "Bearer token123"},
                    timeout_ms=7000,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            project=project,
            phone_number=phone_number,
            timezone=timezone,
            voice_id=voice_id,
            audio_format=audio_format,
            audio_speed=audio_speed,
            welcome_message=welcome_message,
            system_prompt=system_prompt,
            template_variables=template_variables,
            tools=tools,
            tasks=tasks,
            no_input_poke_sec=no_input_poke_sec,
            no_input_poke_text=no_input_poke_text,
            no_input_end_conversation_sec=no_input_end_conversation_sec,
            boosted_keywords=boosted_keywords,
            configuration_endpoint=configuration_endpoint,
            request_options=request_options,
        )
        return _response.data

    async def upsert(
        self,
        *,
        name: str,
        project: typing.Optional[str] = None,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[CreateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]] = OMIT,
        tools: typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]] = OMIT,
        tasks: typing.Optional[typing.Sequence[Task]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[CreateAgentRequestConfigurationEndpoint] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsUpsertResponse:
        """
        Upserts an agent by name. If an agent with the same name already exists, it will be updated. Otherwise, it will be created.

        Parameters
        ----------
        name : str
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        project : typing.Optional[str]
            The name of the project containing the agent.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[CreateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, CreateAgentRequestTemplateVariablesValue]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[CreateAgentRequestToolsItem]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[Task]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[CreateAgentRequestConfigurationEndpoint]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsUpsertResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import (
            AsyncPhonic,
            CreateAgentRequestConfigurationEndpoint,
            CreateAgentRequestTemplateVariablesValue,
        )

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.upsert(
                name="support-agent",
                phone_number="assign-automatically",
                timezone="America/Los_Angeles",
                voice_id="sarah",
                audio_speed=1.0,
                welcome_message="Hi {{customer_name}}. How can I help you today?",
                system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
                template_variables={
                    "customer_name": CreateAgentRequestTemplateVariablesValue(),
                    "subject": CreateAgentRequestTemplateVariablesValue(
                        default_value="Chess",
                    ),
                },
                tools=[],
                no_input_poke_sec=30,
                no_input_poke_text="Are you still there?",
                boosted_keywords=["Load ID", "dispatch"],
                configuration_endpoint=CreateAgentRequestConfigurationEndpoint(
                    url="https://api.example.com/config",
                    headers={"Authorization": "Bearer token123"},
                    timeout_ms=7000,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upsert(
            name=name,
            project=project,
            phone_number=phone_number,
            timezone=timezone,
            voice_id=voice_id,
            audio_format=audio_format,
            audio_speed=audio_speed,
            welcome_message=welcome_message,
            system_prompt=system_prompt,
            template_variables=template_variables,
            tools=tools,
            tasks=tasks,
            no_input_poke_sec=no_input_poke_sec,
            no_input_poke_text=no_input_poke_text,
            no_input_end_conversation_sec=no_input_end_conversation_sec,
            boosted_keywords=boosted_keywords,
            configuration_endpoint=configuration_endpoint,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsGetResponse:
        """
        Returns an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to get.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsGetResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.get(
                name_or_id="nameOrId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(name_or_id, project=project, request_options=request_options)
        return _response.data

    async def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsDeleteResponse:
        """
        Deletes an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to delete.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsDeleteResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.delete(
                name_or_id="nameOrId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(name_or_id, project=project, request_options=request_options)
        return _response.data

    async def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[typing.Literal["assign-automatically"]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        voice_id: typing.Optional[str] = OMIT,
        audio_format: typing.Optional[UpdateAgentRequestAudioFormat] = OMIT,
        audio_speed: typing.Optional[float] = OMIT,
        welcome_message: typing.Optional[str] = OMIT,
        system_prompt: typing.Optional[str] = OMIT,
        template_variables: typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValue]] = OMIT,
        tools: typing.Optional[typing.Sequence[UpdateAgentRequestToolsItem]] = OMIT,
        tasks: typing.Optional[typing.Sequence[Task]] = OMIT,
        no_input_poke_sec: typing.Optional[int] = OMIT,
        no_input_poke_text: typing.Optional[str] = OMIT,
        no_input_end_conversation_sec: typing.Optional[int] = OMIT,
        boosted_keywords: typing.Optional[typing.Sequence[str]] = OMIT,
        configuration_endpoint: typing.Optional[UpdateAgentRequestConfigurationEndpoint] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsUpdateResponse:
        """
        Updates an agent by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the agent to update.

        project : typing.Optional[str]
            The name of the project containing the agent. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the agent. Can only contain lowercase letters, numbers and hyphens. Must be unique within the project.

        phone_number : typing.Optional[typing.Literal["assign-automatically"]]

        timezone : typing.Optional[str]
            The timezone of the agent. Used to format system variables like `{{system_time}}`.

        voice_id : typing.Optional[str]
            The voice ID to use.

        audio_format : typing.Optional[UpdateAgentRequestAudioFormat]
            The audio format of the agent.

        audio_speed : typing.Optional[float]
            The audio speed of the agent.

        welcome_message : typing.Optional[str]
            Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.

        system_prompt : typing.Optional[str]
            Instructions for the conversation. Can contain template variables like `{{subject}}`.

        template_variables : typing.Optional[typing.Dict[str, UpdateAgentRequestTemplateVariablesValue]]
            Variables that can be used in the welcome message and the system prompt.

        tools : typing.Optional[typing.Sequence[UpdateAgentRequestToolsItem]]
            Array of built-in or custom tool names to use.

        tasks : typing.Optional[typing.Sequence[Task]]
            Array of task objects with `name` and `description` fields.

        no_input_poke_sec : typing.Optional[int]
            Number of seconds of silence before sending a poke message. `null` disables the poke message.

        no_input_poke_text : typing.Optional[str]
            The message to send after the specified silence.

        no_input_end_conversation_sec : typing.Optional[int]
            Seconds of silence before ending the conversation.

        boosted_keywords : typing.Optional[typing.Sequence[str]]
            These words, or short phrases, will be more accurately recognized by the agent.

        configuration_endpoint : typing.Optional[UpdateAgentRequestConfigurationEndpoint]
            When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsUpdateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic
        from phonic.agents import (
            UpdateAgentRequestConfigurationEndpoint,
            UpdateAgentRequestTemplateVariablesValue,
        )

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.update(
                name_or_id="nameOrId",
                name="updated-support-agent",
                phone_number="assign-automatically",
                timezone="America/Los_Angeles",
                voice_id="sarah",
                audio_speed=1.0,
                welcome_message="Hi {{customer_name}}. How can I help you today?",
                system_prompt="You are an expert in {{subject}}. Be friendly, helpful and concise.",
                template_variables={
                    "customer_name": UpdateAgentRequestTemplateVariablesValue(),
                    "subject": UpdateAgentRequestTemplateVariablesValue(
                        default_value="Chess",
                    ),
                },
                tools=[],
                no_input_poke_sec=30,
                no_input_poke_text="Are you still there?",
                boosted_keywords=["Load ID", "dispatch"],
                configuration_endpoint=UpdateAgentRequestConfigurationEndpoint(
                    url="https://api.example.com/config",
                    headers={"Authorization": "Bearer token123"},
                    timeout_ms=7000,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            name_or_id,
            project=project,
            name=name,
            phone_number=phone_number,
            timezone=timezone,
            voice_id=voice_id,
            audio_format=audio_format,
            audio_speed=audio_speed,
            welcome_message=welcome_message,
            system_prompt=system_prompt,
            template_variables=template_variables,
            tools=tools,
            tasks=tasks,
            no_input_poke_sec=no_input_poke_sec,
            no_input_poke_text=no_input_poke_text,
            no_input_end_conversation_sec=no_input_end_conversation_sec,
            boosted_keywords=boosted_keywords,
            configuration_endpoint=configuration_endpoint,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AgentsCreateResponse,
    AgentsDeleteResponse,
    AgentsGetResponse,
    AgentsListResponse,
    AgentsUpdateResponse,
    AgentsUpsertResponse,
    UpdateAgentRequestAudioFormat,
    UpdateAgentRequestConfigurationEndpoint,
    UpdateAgentRequestTemplateVariablesValue,
    UpdateAgentRequestToolsItem,
)

__all__ = [
    "AgentsCreateResponse",
    "AgentsDeleteResponse",
    "AgentsGetResponse",
    "AgentsListResponse",
    "AgentsUpdateResponse",
    "AgentsUpsertResponse",
    "UpdateAgentRequestAudioFormat",
    "UpdateAgentRequestConfigurationEndpoint",
    "UpdateAgentRequestTemplateVariablesValue",
    "UpdateAgentRequestToolsItem",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.extraction_field import ExtractionField
from .raw_client import AsyncRawExtractionSchemasClient, RawExtractionSchemasClient
from .types.extraction_schemas_create_response import ExtractionSchemasCreateResponse
from .types.extraction_schemas_delete_response import ExtractionSchemasDeleteResponse
from .types.extraction_schemas_get_response import ExtractionSchemasGetResponse
from .types.extraction_schemas_list_response import ExtractionSchemasListResponse
from .types.extraction_schemas_update_response import ExtractionSchemasUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ExtractionSchemasClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawExtractionSchemasClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawExtractionSchemasClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawExtractionSchemasClient
        """
        return self._raw_client

    def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ExtractionSchemasListResponse:
        """
        Returns all extraction schemas in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list extraction schemas for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasListResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.extraction_schemas.list()
        """
        _response = self._raw_client.list(project=project, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        name: str,
        prompt: str,
        fields: typing.Sequence[ExtractionField],
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasCreateResponse:
        """
        Creates a new extraction schema in a project.

        Parameters
        ----------
        name : str
            A name for the extraction schema.

        prompt : str
            Instructions for how to extract data from conversations.

        fields : typing.Sequence[ExtractionField]
            Array of field definitions.

        project : typing.Optional[str]
            The name of the project to create the extraction schema in.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasCreateResponse
            Success response

        Examples
        --------
        from phonic import ExtractionField, Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.extraction_schemas.create(
            name="Appointment details",
            prompt="Dates should be in `9 Apr 2025` format. Prices should be in $150.00 format.",
            fields=[
                ExtractionField(
                    name="Date",
                    type="string",
                    description="The date of the appointment",
                ),
                ExtractionField(
                    name="Copay",
                    type="string",
                    description="Amount of money the patient pays for the appointment",
                ),
                ExtractionField(
                    name="Confirmed as booked",
                    type="bool",
                    description="Is the appointment confirmed as booked?",
                ),
            ],
        )
        """
        _response = self._raw_client.create(
            name=name, prompt=prompt, fields=fields, project=project, request_options=request_options
        )
        return _response.data

    def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasGetResponse:
        """
        Returns an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to get.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasGetResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.extraction_schemas.get(
            name_or_id="nameOrId",
        )
        """
        _response = self._raw_client.get(name_or_id, project=project, request_options=request_options)
        return _response.data

    def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasDeleteResponse:
        """
        Deletes an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to delete.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasDeleteResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.extraction_schemas.delete(
            name_or_id="nameOrId",
        )
        """
        _response = self._raw_client.delete(name_or_id, project=project, request_options=request_options)
        return _response.data

    def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        fields: typing.Optional[typing.Sequence[ExtractionField]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasUpdateResponse:
        """
        Updates an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to update.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            A name for the extraction schema.

        prompt : typing.Optional[str]
            Instructions for how to extract data from conversations.

        fields : typing.Optional[typing.Sequence[ExtractionField]]
            Array of field definitions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasUpdateResponse
            Success response

        Examples
        --------
        from phonic import ExtractionField, Phonic

        client = Phonic(
            token="YOUR_TOKEN",
        )
        client.extraction_schemas.update(
            name_or_id="nameOrId",
            name="Updated appointment details",
            prompt="Updated extraction instructions. Dates should be in `9 Apr 2025` format.",
            fields=[
                ExtractionField(
                    name="Date",
                    type="string",
                    description="The date of the appointment",
                ),
                ExtractionField(
                    name="Time",
                    type="string",
                    description="The time of the appointment",
                ),
            ],
        )
        """
        _response = self._raw_client.update(
            name_or_id, project=project, name=name, prompt=prompt, fields=fields, request_options=request_options
        )
        return _response.data


class AsyncExtractionSchemasClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawExtractionSchemasClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawExtractionSchemasClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawExtractionSchemasClient
        """
        return self._raw_client

    async def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ExtractionSchemasListResponse:
        """
        Returns all extraction schemas in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list extraction schemas for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasListResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extraction_schemas.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project=project, request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        name: str,
        prompt: str,
        fields: typing.Sequence[ExtractionField],
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasCreateResponse:
        """
        Creates a new extraction schema in a project.

        Parameters
        ----------
        name : str
            A name for the extraction schema.

        prompt : str
            Instructions for how to extract data from conversations.

        fields : typing.Sequence[ExtractionField]
            Array of field definitions.

        project : typing.Optional[str]
            The name of the project to create the extraction schema in.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasCreateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic, ExtractionField

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extraction_schemas.create(
                name="Appointment details",
                prompt="Dates should be in `9 Apr 2025` format. Prices should be in $150.00 format.",
                fields=[
                    ExtractionField(
                        name="Date",
                        type="string",
                        description="The date of the appointment",
                    ),
                    ExtractionField(
                        name="Copay",
                        type="string",
                        description="Amount of money the patient pays for the appointment",
                    ),
                    ExtractionField(
                        name="Confirmed as booked",
                        type="bool",
                        description="Is the appointment confirmed as booked?",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name, prompt=prompt, fields=fields, project=project, request_options=request_options
        )
        return _response.data

    async def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasGetResponse:
        """
        Returns an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to get.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasGetResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extraction_schemas.get(
                name_or_id="nameOrId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(name_or_id, project=project, request_options=request_options)
        return _response.data

    async def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasDeleteResponse:
        """
        Deletes an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to delete.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasDeleteResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extraction_schemas.delete(
                name_or_id="nameOrId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(name_or_id, project=project, request_options=request_options)
        return _response.data

    async def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        fields: typing.Optional[typing.Sequence[ExtractionField]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractionSchemasUpdateResponse:
        """
        Updates an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to update.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            A name for the extraction schema.

        prompt : typing.Optional[str]
            Instructions for how to extract data from conversations.

        fields : typing.Optional[typing.Sequence[ExtractionField]]
            Array of field definitions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractionSchemasUpdateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic, ExtractionField

        client = AsyncPhonic(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extraction_schemas.update(
                name_or_id="nameOrId",
                name="Updated appointment details",
                prompt="Updated extraction instructions. Dates should be in `9 Apr 2025` format.",
                fields=[
                    ExtractionField(
                        name="Date",
                        type="string",
                        description="The date of the appointment",
                    ),
                    ExtractionField(
                        name="Time",
                        type="string",
                        description="The time of the appointment",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            name_or_id, project=project, name=name, prompt=prompt, fields=fields, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .conversation_extraction_schema import ConversationExtractionSchema


class ConversationExtraction(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The extraction ID.
    """

    conversation_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The conversation ID.
    """

    schema_: typing_extensions.Annotated[ConversationExtractionSchema, FieldMetadata(alias="schema")]
    result: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    The extracted data.
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if extraction failed.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    When the extraction was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.not_found_error import NotFoundError
from ..types.error import Error
from .types.projects_create_eval_prompt_response import ProjectsCreateEvalPromptResponse
from .types.projects_create_response import ProjectsCreateResponse
from .types.projects_delete_response import ProjectsDeleteResponse
from .types.projects_get_response import ProjectsGetResponse
from .types.projects_list_eval_prompts_response import ProjectsListEvalPromptsResponse
from .types.projects_list_response import ProjectsListResponse
from .types.projects_update_response import ProjectsUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawProjectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[ProjectsListResponse]:
        """
        Returns all projects in a workspace.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ProjectsListResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "projects",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsListResponse,
                    parse_obj_as(
                        type_=ProjectsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ProjectsCreateResponse]:
        """
        Creates a new project in a workspace.

        Parameters
        ----------
        name : str
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ProjectsCreateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "projects",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsCreateResponse,
                    parse_obj_as(
                        type_=ProjectsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ProjectsGetResponse]:
        """
        Returns a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ProjectsGetResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsGetResponse,
                    parse_obj_as(
                        type_=ProjectsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ProjectsDeleteResponse]:
        """
        Deletes a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ProjectsDeleteResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsDeleteResponse,
                    parse_obj_as(
                        type_=ProjectsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        name_or_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        default_agent: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ProjectsUpdateResponse]:
        """
        Updates a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to update.

        name : typing.Optional[str]
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        default_agent : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ProjectsUpdateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            json={
                "name": name,
                "default_agent": default_agent,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsUpdateResponse,
                    parse_obj_as(
                        type_=ProjectsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_eval_prompts(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ProjectsListEvalPromptsResponse]:
        """
        Returns all conversation evaluation prompts for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ProjectsListEvalPromptsResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(id)}/conversation_eval_prompts",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsListEvalPromptsResponse,
                    parse_obj_as(
                        type_=ProjectsListEvalPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_eval_prompt(
        self, id: str, *, name: str, prompt: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ProjectsCreateEvalPromptResponse]:
        """
        Creates a new conversation evaluation prompt for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        name : str
            A useful name for referring to this prompt.

        prompt : str
            Actual evaluation prompt text to evaluate conversations with.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ProjectsCreateEvalPromptResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(id)}/conversation_eval_prompts",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "name": name,
                "prompt": prompt,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsCreateEvalPromptResponse,
                    parse_obj_as(
                        type_=ProjectsCreateEvalPromptResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawProjectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ProjectsListResponse]:
        """
        Returns all projects in a workspace.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ProjectsListResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "projects",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsListResponse,
                    parse_obj_as(
                        type_=ProjectsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ProjectsCreateResponse]:
        """
        Creates a new project in a workspace.

        Parameters
        ----------
        name : str
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ProjectsCreateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "projects",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsCreateResponse,
                    parse_obj_as(
                        type_=ProjectsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ProjectsGetResponse]:
        """
        Returns a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ProjectsGetResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsGetResponse,
                    parse_obj_as(
                        type_=ProjectsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, name_or_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ProjectsDeleteResponse]:
        """
        Deletes a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ProjectsDeleteResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsDeleteResponse,
                    parse_obj_as(
                        type_=ProjectsDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        name_or_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        default_agent: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ProjectsUpdateResponse]:
        """
        Updates a project by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the project to update.

        name : typing.Optional[str]
            The name of the project. Can only contain lowercase letters, numbers and hyphens. Must be unique within the workspace.

        default_agent : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ProjectsUpdateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            json={
                "name": name,
                "default_agent": default_agent,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsUpdateResponse,
                    parse_obj_as(
                        type_=ProjectsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_eval_prompts(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ProjectsListEvalPromptsResponse]:
        """
        Returns all conversation evaluation prompts for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ProjectsListEvalPromptsResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(id)}/conversation_eval_prompts",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsListEvalPromptsResponse,
                    parse_obj_as(
                        type_=ProjectsListEvalPromptsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_eval_prompt(
        self, id: str, *, name: str, prompt: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ProjectsCreateEvalPromptResponse]:
        """
        Creates a new conversation evaluation prompt for a project.

        Parameters
        ----------
        id : str
            The ID of the project.

        name : str
            A useful name for referring to this prompt.

        prompt : str
            Actual evaluation prompt text to evaluate conversations with.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ProjectsCreateEvalPromptResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"projects/{jsonable_encoder(id)}/conversation_eval_prompts",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "name": name,
                "prompt": prompt,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ProjectsCreateEvalPromptResponse,
                    parse_obj_as(
                        type_=ProjectsCreateEvalPromptResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .bad_request_error import BadRequestError
from .conflict_error import ConflictError
from .forbidden_error import ForbiddenError
from .gateway_timeout_error import GatewayTimeoutError
from .not_found_error import NotFoundError

__all__ = ["BadRequestError", "ConflictError", "ForbiddenError", "GatewayTimeoutError", "NotFoundError"]
